/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */ 
package com.andyfrank.mame.ui;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * RomManager manages the list of Roms.
 */
public class RomManager
{  
////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////

  public RomManager()
  {    
  }  
  
////////////////////////////////////////////////////////////////
// Methods
////////////////////////////////////////////////////////////////
  
  /**
   * Add a Rom.
   */
  public void addRom(Rom rom)
  {
    roms.add(rom);
  }  
  
  /**
   * Get the rom at <code>index</code>.
   */
  public Rom getRom(int index)
  {
    return (Rom)roms.get(index);
  }
  
  /**
   * Get the number of Roms.  If favs is
   * true include only favorite roms.
   */
  public int getSize(boolean favs)
  {
    if (!favs) return roms.size();
    
    int count = 0;
    for(int i=0; i<roms.size(); i++)
    {
      Rom r = (Rom)roms.get(i);
      if (r.isFavorite()) count++;
    }
    return count;
  }
  
  /**
   * Get the list of Rom's.
   */
  public Rom[] getRomList(boolean favs)
  {
    if (!favs) return (Rom[])roms.toArray(new Rom[roms.size()]);
    
    Rom[] r = new Rom[getSize(true)];
    for(int i=0,j=0; i<roms.size(); i++)
    {
      Rom temp = (Rom)roms.get(i);
      if (!temp.isFavorite()) continue;
      r[j++] = temp; 
    }
    return r;
  }
  
  /**
   * Run the rom index.
   */
  public void run(int index)
  {
    Rom rom = getRom(index);
    String exe  = Config.getMameHome() + "/mame";
    String game = Config.getMameHome() + "/roms/" + rom.getName();
             
    try
    {
      // TODO - can we rig this to now show that command window?
      // Probelm - new process doesn't get focus otherwise
      String cmd = "cmd.exe /c start " + exe + " " + game;
      if (rom.params != null) cmd += " " + rom.params;
      Runtime.getRuntime().exec(cmd, null, new File(Config.getMameHome()));      
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
  
  /**
   * Load metadata file and assign values.
   */
  public void loadMetaData(String filename)
  {
    // Get defaults from gamelist.txt
    try
    {
      HashMap hash = new HashMap();
      File file = new File("gamelist.txt");
      BufferedReader in = new BufferedReader(new FileReader(file));
      String line = in.readLine();
      while (line != null)
      {
        if (line.length() == 0) break;
        
        int i = line.indexOf(" ");
        String a = line.substring(0, i);
        
        i = line.indexOf("\"");
        String b = line.substring(i+1, line.length()-1); // lose quotes
        hash.put(a, b); 
        
        line = in.readLine();
      }
      
      for(int i=0; i<roms.size(); i++)
      {
        Rom r = (Rom)roms.get(i);
        String s = (String)hash.get(r.getName());
        if (s != null) r.setDefaultName(s);
      }
      
      SAXParserFactory factory = SAXParserFactory.newInstance();
      SAXParser saxParser = factory.newSAXParser();
      saxParser.parse(new File(filename), new Handler());
      Collections.sort(roms, new Sort());
    } 
    catch (Exception e) { e.printStackTrace(); }    
  }  

////////////////////////////////////////////////////////////////
// Handler
////////////////////////////////////////////////////////////////

  class Handler extends DefaultHandler
  {
    public void startElement(String ns, String sName, String qName, Attributes attrs)
      throws SAXException
    {
      if (qName.equals("rom"))
      {
        String n = attrs.getValue("name");
        String d = attrs.getValue("display");
        String p = attrs.getValue("params");
        // TODO - get rid of hidden
        String h = attrs.getValue("hidden");
        String f = attrs.getValue("fav");
        
        if (n != null)
        {
          Rom rom = findRom(n);
          if (rom == null) return;
          if (h != null) rom.setFavorite(h.equals("true") ? true : false);
          if (f != null) rom.setFavorite(f.equals("true") ? true : false);
          if (d != null) rom.setDisplayName(d);
          if (p != null) rom.setParams(p);
        }
      }
    }
  }  
  
////////////////////////////////////////////////////////////////
// Sort
////////////////////////////////////////////////////////////////

  class Sort implements Comparator
  {
    public int compare(Object o1, Object o2)
    {
      Rom a = (Rom)o1;
      Rom b = (Rom)o2;      
      return a.getDisplayName().compareToIgnoreCase(b.getDisplayName());
    }
  }
  
////////////////////////////////////////////////////////////////
// Util
////////////////////////////////////////////////////////////////
  
  private Rom findRom(String name)
  {
    for(int i=0; i<roms.size(); i++)
    {
      Rom rom = (Rom)roms.get(i);
      if (rom.getName().equals(name)) return rom;
    }
    return null;
  }

////////////////////////////////////////////////////////////////
// Output
////////////////////////////////////////////////////////////////

  public void save()
  {
    try
    {
      PrintWriter out = new PrintWriter(new FileOutputStream("rominfo.xml"));
      out.println("<roms>");
      
      for(int i=0; i<roms.size(); i++)
      {
        // Don't output if rom has default settings
        Rom r = (Rom)roms.get(i);
        if (!r.isFavorite() && r.displayName == null) continue;
        
        StringBuffer buf = new StringBuffer();
        buf.append("  <rom name=\"").append(r.getName()).append("\"");
        
        if (r.displayName != null) 
          buf.append(" display=\"").append(r.displayName).append("\"");
        
        if (r.params != null) 
          buf.append(" params=\"").append(r.params).append("\"");
          
        if (r.isFavorite()) 
          buf.append(" fav=\"").append("true").append("\"");
        
        buf.append("/>");      
        out.println(buf.toString());      
      }
      
      out.println("</roms>");
      out.flush();
      out.close();
    }
    catch (Exception e) { e.printStackTrace(); }
  }
  
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////

  private ArrayList roms = new ArrayList();
}

